@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  *
  * Author: Ray Strode <rstrode@redhat.com>
 */
@@ -27,9 +27,13 @@
 #include <gio/gio.h>
 
 #include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <gtk/gtk.h>
 
 #include <cairo.h>
+#include <cairo-xlib.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include "gnome-bg.h"
@@ -61,7 +65,10 @@ enum {
 
 static guint signals[NUMBER_OF_SIGNALS] = { 0 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GnomeBGCrossfade, gnome_bg_crossfade, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GnomeBGCrossfade, gnome_bg_crossfade, G_TYPE_OBJECT)
+#define GNOME_BG_CROSSFADE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o),\
+			                   GNOME_TYPE_BG_CROSSFADE,\
+			                   GnomeBGCrossfadePrivate))
 
 static void
 gnome_bg_crossfade_set_property (GObject      *object,
@@ -188,12 +195,14 @@ gnome_bg_crossfade_class_init (GnomeBGCrossfadeClass *fade_class)
 					  G_SIGNAL_RUN_LAST, 0, NULL, NULL,
 					  g_cclosure_marshal_VOID__OBJECT,
 					  G_TYPE_NONE, 1, G_TYPE_OBJECT);
+
+	g_type_class_add_private (gobject_class, sizeof (GnomeBGCrossfadePrivate));
 }
 
 static void
 gnome_bg_crossfade_init (GnomeBGCrossfade *fade)
 {
-	fade->priv = gnome_bg_crossfade_get_instance_private (fade);
+	fade->priv = GNOME_BG_CROSSFADE_GET_PRIVATE (fade);
 
 	fade->priv->fading_surface = NULL;
 	fade->priv->end_surface = NULL;
@@ -361,11 +370,34 @@ animations_are_disabled (GnomeBGCrossfade *fade)
 	return !are_enabled;
 }
 
+static void
+send_root_property_change_notification (GnomeBGCrossfade *fade)
+{
+        long zero_length_pixmap;
+
+        /* We do a zero length append to force a change notification,
+         * without changing the value */
+        XChangeProperty (GDK_WINDOW_XDISPLAY (fade->priv->window),
+                         GDK_WINDOW_XID (fade->priv->window),
+                         gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"),
+                         XA_PIXMAP, 32, PropModeAppend,
+                         (guchar *) &zero_length_pixmap, 0);
+}
+
 static void
 draw_background (GnomeBGCrossfade *fade)
 {
 	if (gdk_window_get_window_type (fade->priv->window) == GDK_WINDOW_ROOT) {
-		g_warning ("Crossfade is not supported on ROOT window!");
+                XClearArea (GDK_WINDOW_XDISPLAY (fade->priv->window),
+                            GDK_WINDOW_XID (fade->priv->window),
+                            0, 0,
+                            gdk_window_get_width (fade->priv->window),
+                            gdk_window_get_height (fade->priv->window),
+                            False);
+
+                send_root_property_change_notification (fade);
+
+		gdk_flush ();
 	} else {
 		gdk_window_invalidate_rect (fade->priv->window, NULL, FALSE);
 		gdk_window_process_updates (fade->priv->window, FALSE);
