@@ -1056,7 +1056,7 @@ gnome_bg_get_pixmap_size (GnomeBG   *bg,
  * @window: 
  * @width: 
  * @height:
- * @is_root:
+ * @root:
  *
  * Create a surface that can be set as background for @window. If @is_root is
  * TRUE, the surface created will be created by a temporary X server connection
@@ -1068,7 +1068,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 		 	 GdkWindow   *window,
 			 int	     width,
 			 int	     height,
-			 gboolean     is_root)
+			 gboolean     root)
 {
 	int pm_width, pm_height;
 	cairo_surface_t *surface;
@@ -1088,7 +1088,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 	/* has the side effect of loading and caching pixbuf only when in tile mode */
 	gnome_bg_get_pixmap_size (bg, width, height, &pm_width, &pm_height);
 	
-	if (is_root) {
+	if (root) {
 		surface = make_root_pixmap (gdk_window_get_screen (window),
 					   pm_width, pm_height);
 	}
@@ -1111,7 +1111,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 		
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
 					 width, height);
-		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), is_root);
+		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), root);
 		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
 		pixbuf_average_value (pixbuf, &average);
 		g_object_unref (pixbuf);
