@@ -44,6 +44,11 @@
 #define CONFIG_INTENDED_BASENAME "monitors.xml"
 #define CONFIG_BACKUP_BASENAME "monitors.xml.backup"
 
+/* Look for DPI_FALLBACK in:
+ * http://git.gnome.org/browse/gnome-settings-daemon/tree/plugins/xsettings/gsd-xsettings-manager.c
+ * for the reasoning */
+#define DPI_FALLBACK 96.0
+
 /* In version 0 of the config file format, we had several <configuration>
  * toplevel elements and no explicit version number.  So, the filed looked
  * like
@@ -1817,18 +1822,14 @@ crtc_assignment_new (GnomeRRScreen *screen, GnomeRROutputInfo **outputs, GError
     {
 	int width, height;
 	int min_width, max_width, min_height, max_height;
-	int required_pixels, min_pixels, max_pixels;
 
 	get_required_virtual_size (assignment, &width, &height);
 
 	gnome_rr_screen_get_ranges (
 	    screen, &min_width, &max_width, &min_height, &max_height);
-
-	required_pixels = width * height;
-	min_pixels = min_width * min_height;
-	max_pixels = max_width * max_height;
-
-	if (required_pixels < min_pixels || required_pixels > max_pixels)
+    
+	if (width < min_width || width > max_width ||
+	    height < min_height || height > max_height)
 	{
 	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_BOUNDS_ERROR,
 			 /* Translators: the "requested", "minimum", and
@@ -1928,8 +1929,8 @@ crtc_assignment_apply (CrtcAssignment *assign, guint32 timestamp, GError **error
      *
      * Firefox and Evince apparently believe what X tells them.
      */
-    width_mm = (width / 96.0) * 25.4 + 0.5;
-    height_mm = (height / 96.0) * 25.4 + 0.5;
+    width_mm = (width / DPI_FALLBACK) * 25.4 + 0.5;
+    height_mm = (height / DPI_FALLBACK) * 25.4 + 0.5;
 
     if (success)
     {
