@@ -947,11 +947,11 @@ rr_screen_weak_notify_cb (gpointer data, GObject *where_the_object_was)
 
 /**
  * gnome_rr_screen_new:
- * Creates a unique #GnomeRRScreen instance for the specified @screen.
- *
  * @screen: the #GdkScreen on which to operate
  * @error: will be set if XRandR is not supported
  *
+ * Creates a unique #GnomeRRScreen instance for the specified @screen.
+ *
  * Returns: a unique #GnomeRRScreen instance, specific to the @screen, or NULL
  * if this could not be created, for instance if the driver does not support
  * Xrandr 1.2.  Each #GdkScreen thus has a single instance of #GnomeRRScreen.
@@ -998,13 +998,13 @@ gnome_rr_screen_set_size (GnomeRRScreen *screen,
 
 /**
  * gnome_rr_screen_get_ranges:
- *
- * Get the ranges of the screen
  * @screen: a #GnomeRRScreen
  * @min_width: (out): the minimum width
  * @max_width: (out): the maximum width
  * @min_height: (out): the minimum height
  * @max_height: (out): the maximum height
+ *
+ * Get the ranges of the screen
  */
 void
 gnome_rr_screen_get_ranges (GnomeRRScreen *screen,
@@ -1869,7 +1869,7 @@ out:
 }
 
 /**
- * gnome_rr_screen_get_output_by_id:
+ * gnome_rr_screen_get_output_by_name:
  *
  * Returns: (transfer none): the output identified by @name
  */
@@ -1921,7 +1921,6 @@ _gnome_rr_output_name_is_laptop (const char *name)
 	strstr (name, "Lvds") ||
 	strstr (name, "LCD")  ||  /* ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye. */
 	strstr (name, "eDP")  ||  /* eDP is for internal laptop panel connections */
-	strstr (name, "DFP")  ||  /* DFP is also an internal laptop display */
 	strstr (name, "default")) /* Finally, NVidia and all others that don't bother to do RANDR properly */
         return TRUE;
 
